function [alpha,MFS] = mfs_kmean(im,varargin)
%
%
%[alpha,Mfs] = mfs_kmean(im) compute the MFS and the vector of central points
%                             for input im with default  setting
%
%[alpha,Mfs] = mfs_kmean(im,ind_num)  computes the MFS with ind_num density levels 
%
%[alpha,Mfs] = mfs_kmean(im,ind_num,f_num) computes the MFS of dimension f_num for input im 
%                             with ind_num density levels.
%[alpha,Mfs] = mfs_kmean(im,ind_num,f_num,ite_num) computes the MFS of dimension f_num for input image im
%                                  using ite_num level iterations in the fractal dimension 
%                                  estimation and using ind_num level
%                                  iterations in the density estimation.
%
%Author: Yong Xu, Hui Ji
%Date: Apr 24, 2007



switch nargin,
     case 2,
        ind_num = varargin{1};  %density counting levels
        f_num = 26;  %the dimension of MFS
        ite_num = 3; % iteration levels in estimating fractal dimension
    case 3,
        ind_num = varargin{1};
        f_num = varargin{2};
        ite_num = 3;
    case 4,    
        ind_num = varargin{1};
        f_num = varargin{2};
        ite_num = varargin{3};
    otherwise,
        ind_num = 1; %density counting levels
        f_num = 26;  %the dimension of MFS
        ite_num = 3; %Box counting levels  
end;

% Preprocessing: if IM is a color image convert it to a gray image 
if (length(size(im)) == 3),
    display('Warning: Only work on grayscale image, converted to grayscale image');
    im = rgb2gray(im);
end;
im = double(im);

%Using [0..255] to denote the intensity profile of the image
grayscale_box =[0, 255];

%Preprocessing: default intensity value of image ranges from 0 to 255
if max(abs(im))< 1,
    im = im * grayscale_box(2);
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%Estimating density function of the image
%%%%% by solving least squares for D in  the equation  
%%%%% log10(bw) = D*log10(c) + b 
r = 1/max(size(im));
c = (1:ind_num)*r;
c = log10(c);
bw(:,:,1) = im+1;
for k = 2:ind_num,
    mask = fspecial('gaussian',k,k/2);
    bw(:,:,k) = conv2(im+1,mask,'same') * k^2;
end;
bw = log10(bw);
n1 = c(1)*c(1);
n2 = bw(:,:,1)*c(1);

for k = 2:ind_num,
    n1 = n1+c(k)*c(k);
    n2(:,:) = n2(:,:) + bw(:,:,k).*c(k);
end;
% D = n2/n1;
if (ind_num >1),
    D  = (n2-sum(c)*sum(bw,3))/(n1 -sum(c)*sum(c));
%    b = (-n2*sum(c) + n1*sum(bw,3))/(n1 -sum(c)*sum(c));
end;



% renormalize the density
clear bw;

if (ind_num > 1)
    if min(D(:))<0
      D = -D;
    end;
%   D = log10(D+0.01);
    D = grayscale_box(2)*(D - min(D(:))) / (max(D(:)) - min(D(:))) + grayscale_box(1);
else
    D = im;
end;


%Clustering the density
IM = zeros(size(im));
Max_Num = length(unique(D));
f_num = min(Max_Num,f_num);
seed = linspace(min(D(:)),max(D(:)),f_num+2)';
seed = seed(2:end-1);
[f_idx,f] = kmeans(D(:),f_num,'Start',seed,'Maxiter',50,'EmptyAction','singleton');
for k = 1:f_num,
    IM(find(f_idx==k)) = f(k);
end


%Constructing the filter for approximating log fitting
r = max(size(IM));
c = zeros(1,ite_num);
c(1) = 1;
for k = 2:ite_num,
    c(k) = c(k-1)/k;
end;
c = c / sum(c);

%Construct level sets
Idx_IM = zeros(size(IM));
for k = 1:f_num,
    idx = find(IM == f(k));
    Idx_IM(idx) = k;
end;
Idx_IM;

%Estimate MFS by box-counting
num = zeros(1,ite_num);
MFS = zeros(1,f_num);
for k = 1:f_num,
    idx = find(Idx_IM == k);
    IM = zeros(size(im));
    IM(idx) = 1;
    temp = max(length(idx),1);
    num(1) = log10(temp)/log10(r);    
    for j = 2:ite_num,
        mask = ones(j,j);
        bw = conv2(IM,mask,'same');
        bw = bw(1:j:end,1:j:end);
        idx = find(bw>0);
        temp = max(length(idx),1);
        num(j) = log10(temp)/log10(r/j);
    end;
    MFS(k) = sum(c.*num);
end;

[junk,idx] = sort(f);
alpha = f(idx);
MFS = MFS(idx);
idx = find(~isnan(alpha));
alpha = alpha(idx)';
MFS = MFS(idx);



